from django.test import TestCase

from ovp_uploads.models import UploadedImage

from uuid import UUID


def is_valid_uuid(uuid_to_test, version=4):
  try:
    uuid_obj = UUID(uuid_to_test, version=version)
  except: # pragma: no cover
    return False

  return str(uuid_obj) == uuid_to_test

class UploadedImageModelTestCase(TestCase):
  def test_str_return_uuid(self):
    """Assert that image model __str__ method returns uuid"""
    img = UploadedImage()
    img.save()

    uuid = img.__str__()

    self.assertTrue(is_valid_uuid(uuid))



from django.test import TestCase
from django.conf import settings

from rest_framework.reverse import reverse
from rest_framework.test import APIClient
from rest_framework.test import APIRequestFactory

from ovp_uploads.models import UploadedImage
from ovp_uploads.serializers import UploadedImageSerializer
from django.contrib.auth import get_user_model

from PIL import Image

from tempfile import NamedTemporaryFile


class UploadedImageSerializerTestCase(TestCase):
  def test_image_urls(self):
    """Assert that image object returns url"""
    user = get_user_model().objects.create_user('test_image_urls@test.com', 'validpassword')

    client = APIClient()
    client.force_authenticate(user=user)

    image = Image.new('RGB', (100, 100))
    tmp_file = NamedTemporaryFile()
    image.save(tmp_file, format="JPEG")
    tmp_file.seek(0) # otherwise we start reading at the end

    data = {
      'image': tmp_file
    }

    response = client.post(reverse('upload-images-list'), data, format="multipart")

    self.assertTrue(response.status_code == 201)

    img_id = response.data['id']
    img = UploadedImage.objects.get(pk=img_id)

    factory = APIRequestFactory()
    request = factory.post('/')
    serializer = UploadedImageSerializer(instance=img, context={'request': request})

    media_path = getattr(settings, "MEDIA_URL")
    if media_path == "":
      media_path = "/"

    test_url = 'http://testserver{media_path}user-uploaded/images'.format(media_path=media_path)

    self.assertTrue(test_url in serializer.get_image_url(img))
    self.assertTrue(test_url in serializer.get_image_small_url(img))
    self.assertTrue(test_url in serializer.get_image_medium_url(img))
    self.assertTrue(test_url in serializer.get_image_large_url(img))

    self.assertTrue(img.uuid in serializer.get_image_url(img))
    self.assertTrue(img.uuid in serializer.get_image_small_url(img))
    self.assertTrue(img.uuid in serializer.get_image_medium_url(img))
    self.assertTrue(img.uuid in serializer.get_image_large_url(img))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-06-13 14:24
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_uploads', '0005_auto_20170522_2006'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='imagegalery',
            options={'verbose_name': 'image gallery', 'verbose_name_plural': 'image galleries'},
        ),
    ]



import os
from io import BytesIO
from tempfile import TemporaryFile
from django.core.files import uploadedfile
from django.core.files.base import ContentFile

from PIL import Image


def perform_image_crop(image_obj, crop_rect=None):
  img_ext = os.path.splitext(image_obj.name)[1][1:].upper()
  img_ext = 'JPEG' if img_ext=='JPG' else img_ext
  if crop_rect is None:
    return image_obj

  image = BytesIO(image_obj.read())

  base_image = Image.open(image)
  tmp_img,tmp_file = base_image.crop(crop_rect), BytesIO()
  tmp_img.save(tmp_file, format=img_ext)

  tmp_file = ContentFile(tmp_file.getvalue())
  return uploadedfile.InMemoryUploadedFile(
    tmp_file, None, image_obj.name, image_obj.content_type, tmp_file.tell, None
    )


from django.conf import settings
from ovp_uploads.models import UploadedImage

from rest_framework import serializers

#
# GCS is actually the default ovp's bucket, so, lets try to optimize for it
# As images *never* changes urls, we can concat domain, bucket and url to obtain a full qualified
# absolute uri, preventing a #get_blog for each image
#
# TODO: It looks like images are stored with absolute urls
#
if hasattr(settings, 'GCS_BUCKET'):
  GCS_BASE_URI = str.join('/', ('https://storage.googleapis.com', settings.GCS_BUCKET))
  def build_absolute_uri(req, image):
    return image.url
    #return str.join('/', (GCS_BASE_URI, image.url)) if image else None

else:
  def build_absolute_uri(req, image):
    return req.build_absolute_uri(image.url) if image else None

class UploadedImageSerializer(serializers.ModelSerializer):
  image_url = serializers.SerializerMethodField()
  image_small_url = serializers.SerializerMethodField()
  image_medium_url = serializers.SerializerMethodField()
  image_large_url = serializers.SerializerMethodField()

  class Meta:
    model = UploadedImage
    fields = ('id', 'user', 'image', 'image_url', 'image_small_url', 'image_medium_url', 'image_large_url')
    read_only_fields = ('image_small', 'image_medium', 'image_large')
    extra_kwargs = {'image': {'write_only': True}, 'crop_rect': {'write_only': True}}

  def get_image_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image)

  def get_image_small_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image_small)

  def get_image_medium_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image_medium)

  def get_image_large_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image_large)

class ImageGallerySerializer(UploadedImageSerializer):
  name = serializers.CharField(read_only=True)
  category = serializers.CharField(read_only=True)
  class Meta:
    model = UploadedImage
    read_only_fields = ('image_small', 'image_medium', 'image_large')
    fields = ('id', 'image_url', 'image_small_url', 'image_medium_url', 'image_large_url', 'name', 'category')


import json

from ovp_uploads.models import UploadedImage
from ovp_uploads.serializers import UploadedImageSerializer, ImageGallerySerializer

from rest_framework import mixins
from rest_framework import viewsets
from rest_framework import response
from rest_framework import status

from .helpers import perform_image_crop


class UploadedImageViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):
  queryset = UploadedImage.objects.all()
  serializer_class = UploadedImageSerializer

  def create(self, request, *args, **kwargs):
    upload_data = {}

    if request.data.get('image', None):
      upload_data['image'] = request.data.get('image')

    upload_header = request.META.get('HTTP_X_UNAUTHENTICATED_UPLOAD', None)
    is_authenticated = request.user.is_authenticated()

    if request.data.get('crop_rect', None):
      crop_rect = request.data.get('crop_rect')
      if isinstance(crop_rect, str):
        crop_rect = json.loads(crop_rect)
      upload_data['image'] = perform_image_crop(upload_data['image'], crop_rect)
      request.FILES['image'] = upload_data['image']

    if is_authenticated or upload_header:
      if upload_header:
        upload_data['user'] = None

      if is_authenticated:
        upload_data['user'] = request.user.id

      serializer = self.get_serializer(data=upload_data)

      if serializer.is_valid():
        self.object = serializer.save()
        headers = self.get_success_headers(serializer.data)
        return response.Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

      return response.Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    return response.Response(status=status.HTTP_401_UNAUTHORIZED)


class ImageGalleryViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
  queryset = UploadedImage.objects.filter(category__isnull=False)
  serializer_class = ImageGallerySerializer

  def get_queryset(self):
    queryset = self.queryset
    category = self.request.query_params.get('category', None)
    if category is not None:
      queryset = queryset.filter(category=category)
    return queryset


from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _

from ovp_uploads.models import UploadedImage


class UploadedImageAdmin(admin.ModelAdmin):
  fields = [
  	'id', 'image', 'image_small', 'image_medium', 'image_large'
  ]

  list_display = [
  	'id', 'image', 'user'
  ]

  list_filter = []

  list_editable = []

  search_fields = [
  	'id', 'user__name', 'user__email'
  ]

  readonly_fields = [
  	'id', 'image_small', 'image_medium', 'image_large'
  ]

  raw_id_fields = [
  	'user'
  ]



class ImageGalery(UploadedImage):
  class Meta:
    proxy = True
    verbose_name = _('image gallery')
    verbose_name_plural = _('image galleries')

class ImageGaleryAdmin(admin.ModelAdmin):
  fields = [
    'id', 'name', 'category', 'image', 'image_small', 'image_medium', 'image_large'
  ]

  list_display = [
    'id', 'name', 'category', 'image'
  ]

  list_filter = ['category']

  list_editable = []

  search_fields = [
    'id', 'name'
  ]

  readonly_fields = [
    'id', 'image_small', 'image_medium', 'image_large'
  ]
  def get_queryset(self, request):
    return super().get_queryset(request).filter(category__isnull=False)


admin.site.register(UploadedImage, UploadedImageAdmin)
admin.site.register(ImageGalery, ImageGaleryAdmin)


# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-05-22 20:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_uploads', '0004_add_category_and_name_to_images'),
    ]

    operations = [
        migrations.AlterField(
            model_name='uploadedimage',
            name='category',
            field=models.CharField(blank=True, choices=[('organization', 'Organization'), ('project', 'Project')], default=None, max_length=24, null=True, verbose_name='Category'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-05-19 20:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_uploads', '0003_uploadedimage_uuid'),
    ]

    operations = [
        migrations.CreateModel(
            name='ImageGalery',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('ovp_uploads.uploadedimage',),
        ),
        migrations.AddField(
            model_name='uploadedimage',
            name='category',
            field=models.CharField(blank=True, choices=[('cover', 'Cover'), ('logo', 'Logo')], default=None, max_length=24, null=True, verbose_name='Category'),
        ),
        migrations.AddField(
            model_name='uploadedimage',
            name='name',
            field=models.CharField(blank=True, default=None, max_length=64, null=True, verbose_name='Name'),
        ),
    ]
